#include "biu/ContactEnergyFunction.hh"
#include "biu/assertbiu.hh"

namespace biu
{

	ContactEnergyFunction::ContactEnergyFunction(
				const Alphabet* const _alphabet, 
				const EnergyMatrix* const _energyMat) 
		: alphabet(_alphabet), energyMat(_energyMat)
	{
		assertbiu(	alphabet->getAlphabetSize() == energyMat->numRows() 
				&& alphabet->getAlphabetSize() == energyMat->numColumns(),
				"energy matrix has the wrong size for this alphabet size");
	}
	
	ContactEnergyFunction::~ContactEnergyFunction()
	{
	}
	
	double 
	ContactEnergyFunction::getContactEnergy( 
							const Alphabet::AlphElem& first, 
							const Alphabet::AlphElem& second) const {
		
	return (*energyMat)[alphabet->getIndex(first)][alphabet->getIndex(second)];
	}
	
	
	bool 
	ContactEnergyFunction::operator == (const ContactEnergyFunction& cef2) const {
		if (this == &cef2)
			return true;
		return	*alphabet == *(cef2.alphabet)
				&& *energyMat == *(energyMat);
	}
	
	bool 
	ContactEnergyFunction::operator != (const ContactEnergyFunction& cef2) const {
		return	!(this->operator ==(cef2));
	}
	

} // namespace biu
