#ifndef LATTICEDESCRIPTORCUB_HH_
#define LATTICEDESCRIPTORCUB_HH_


#include "LatticeDescriptor.hh"
#include <cstdlib>

namespace biu
{

		/**
		 * Lattice property handler for the integer cubic lattice.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class LatticeDescriptorCUB : public LatticeDescriptor
	{
	protected:

			// abstract function implementation
		virtual unsigned int getNeighborDataSize() const;
		virtual const NeighborData *getNeighborData() const;
		virtual unsigned int getAutomorphismDataSize() const;
		virtual const AutomorphismData *getAutomorphismData() const;
	public:
	
		LatticeDescriptorCUB();
		LatticeDescriptorCUB(const LatticeDescriptorCUB& toCopy);
		virtual ~LatticeDescriptorCUB();
			//! Returns whether or not two points in the lattice are neighbored
			//! using a specialised implementation for the cubic lattice
			//! @param first the first point
			//! @param second the second point
			//! @return true if (second-first) is element of the neighborhood,
			//!         false otherwise
		virtual bool areNeighbored( const IntPoint &first, 
									const IntPoint &second ) const;
	};

}


namespace biu
{
		/*! 
		 * A specialized LatticeNeighborhood to handle the
		 * the NeighborVector objects of the cubic lattice and the 
		 * access to them.
		 */
	class LatticeNeighborhoodCUB : public LatticeNeighborhood
	{
	public:
		LatticeNeighborhoodCUB(	const MoveAlphabet* moveAlph, 
								const NeighSet& neighbors)
		 :	LatticeNeighborhood(moveAlph, neighbors) 
		{
		}
		
		LatticeNeighborhoodCUB(	const LatticeNeighborhood& nh ) 
		:	LatticeNeighborhood( nh )
		{
		}
		
								
		virtual ~LatticeNeighborhoodCUB()
		{}
			//! Returns whether or not a vector v belongs to the neighborhood.
		virtual bool isElement( const IntPoint& v) const {
			int sum = abs(v.getX());
			return	sum < 2 
					&& (sum+=abs(v.getY())) < 2 
					&& (sum+=abs(v.getZ())) == 1;
/*
			return (v.getX()* v.getX() + v.getY()*v.getY() + v.getZ()*v.getZ()) 
					== 1;
			return (abs( v.getX() + v.getY() + v.getZ() ) == 1) && 
			((abs(v.getX()) == 1 && v.getY() == 0 && v.getZ() == 0) ||
			(v.getX() == 0 && 
				((abs(v.getY()) == 1 && v.getZ() == 0) ||
				 (v.getY() == 0 && abs(v.getZ()) == 1))) 
			);
*/
		}
	};

} // namespace biu


#endif /*LATTICEDESCRIPTORCUB_HH_*/
